import java.io.File;
import java.io.FileOutputStream;
import java.lang.StringBuffer;
import java.util.regex.*;
import org.pd.parser.*;

/**
 * Only dumps the files it touches;
 * just move them back into the input directory.
 */
public class Massager extends RuleReader {
	private static final String DEFAULT_OUTDIR = "./output";
	private static Pattern groupPattern = Pattern.compile("%\\d+?%");

	private File outDir;

	private static boolean verbose;

	public Massager(File defFile, File inDir) {
		this(defFile, inDir, new File(DEFAULT_OUTDIR));
	}

	public Massager(File defFile, File inDir, File outDir) {
		super(defFile, inDir);

		if (!outDir.exists())
			outDir.mkdirs();

		this.outDir = outDir;
	}

	public static void main(String[] args) {
		Massager m;

		if (args.length < 2)
			fatal("usage: Massager deffile inputdir [outputdir]");

		verbose = System.getenv("VERBOSE") != null;
		if (args.length > 2)
			m = new Massager(new File(args[0]), new File(args[1]), new File(args[2]));
		else
			m = new Massager(new File(args[0]), new File(args[1]));

		m.parse();
		m.dump();
	}

	// match the rule all over again; would be marginally
	// faster if we overrode parseRule() instead, but this
	// is a cleaner way of doing it, methinks.
	public void handleLine(String line, Matcher m) {
		Pattern p = m.pattern();
		SourceFile sf = getSource(getCurrentFile());
		String[] pieces = line.split(" ", 2);

		if (pieces[0].equalsIgnoreCase("define")) {
			pieces = line.split(" ");
			if (pieces.length != 3)
				fatal("Invaild definition line: " + line);

			int index = -1;
			try {
				index = Integer.parseInt(pieces[2]);
			} catch (Exception e) {
				fatal("Couldn't parse integer: " + pieces[2] + ": " + e);
			}

			if (index < 1 || index > m.groupCount())
				fatal("Index out of range " + index + ": " + line);

			debug("Storing variable " + pieces[1] + " as: " + m.group(index));
			setVariable(pieces[1], m.group(index));
			return;
		}
		
		if (!pieces[0].equalsIgnoreCase("replace"))
			fatal("Unknown extraction type: " + line);

		debug("Pattern = " + p);
		do {
			Matcher mm = p.matcher(sf.getContents());
			StringBuffer sb = new StringBuffer(sf.getContents());

			if (!mm.find())
				break;

			int startIndex = mm.start();
			debug("  match from " + mm.start() + " to " + mm.end());
			String matched = sb.substring(startIndex, mm.end());
			debug("  matched text: " + matched);
			String rep = expandVariables(expandGroups(pieces[1], mm));
			debug("  replacement: " + rep);
			sb.replace(startIndex, startIndex + matched.length(), rep);

			sf.setContents(sb.toString());
		} while (true);

		setSource(getCurrentFile(), sf);
	}

	// very ugly way to do this, but it works.
	public void addDeclaration(String decl) {
		SourceFile sf = getSource(getCurrentFile());
		StringBuffer sb = new StringBuffer(sf.getContents());
		sb.insert(sb.lastIndexOf("}"), "    " + decl + "\n");
		sf.setContents(sb.toString());
		setSource(getCurrentFile(), sf);
	}

	// useless to us
	public void handleExtraction(String type, String name, String matched) { }

	public String expandGroups(String line, Matcher m) {
		String tmp = new String(line);

		do {
			Matcher mm = groupPattern.matcher(tmp);
			int index = -1;

			if (!mm.find())
				break;

			String var = tmp.substring(mm.start() + 1, mm.end() - 1);
			try {
				index = Integer.parseInt(var);
			} catch (Exception e) {
				fatal("Couldn't parse integer: " + var + ": " + e);
			}

			if (index < 1 || index > m.groupCount())
				fatal("Indexed variable out of bounds: " + index + ": " + line);

			String val = m.group(index);
			if (val != null)
				tmp = tmp.replaceAll("%" + var + "%", val);
			else
				fatal("Error extracting group " + index + ": " + line);
		} while (true);

		return tmp;
	}

	public void dump() {
		for (SourceFile sf : getSources().values()) {
			try {
				FileOutputStream fos = new FileOutputStream(new File(outDir, sf.getName()));
				fos.write(sf.getContents().getBytes());
				fos.close();
			} catch (Exception e) { fatal("Exception writing output: " + e); }
		}
	}

	public static void debug(String s) {
		if (verbose)
			System.out.println(s);
	}

	public static void fatal(String err) {
		System.err.println(err);
		System.exit(1);
	}
}
