import java.util.ArrayList;
import java.util.HashMap;
import java.util.Map;
import java.util.regex.*;
import org.apache.bcel.classfile.*;
import org.apache.bcel.generic.*;
import org.saev.deob.Application;

public class Packager {
	private static final String DEFAULT_OUTFILE = "./packaged.jar";

	private Application app;
	private String outFile;

	private static boolean verbose;

	private ArrayList<JavaClass> finalClasses = new ArrayList<JavaClass>();
	private HashMap<String, String> classes = new HashMap<String, String>();

	public Packager(String inFile) {
		this(inFile, DEFAULT_OUTFILE);
	}

	public Packager(String inputFile, String outFile) {
		this.outFile = outFile;
		app = new Application(inputFile);
	}

	public static void main(String[] args) {
		Packager r;

		if (args.length < 1)
			fatal("Usage: Packager inputJar [outputJar]");

		verbose = System.getenv("VERBOSE") != null;
		if (args.length > 1)
			r = new Packager(args[0], args[1]);
		else
			r = new Packager(args[0]);

		r.rename();
		r.save();
	}

	public void rename() {
		debug("-- Beginning packaging process --");

		// first task, learn all of the class names
		for (ClassGen cg : app.getClasses()) {
			String name = cg.getClassName();
			String pathed = name.replaceAll("\\.", "/");
			debug("Storing " + pathed + " => rs/" + pathed);
			classes.put(pathed, "rs/" + pathed);
			debug("Renaming class " + name + " => rs." + name);
			cg.setClassName("rs." + name);
		}

		for (ClassGen cg : app.getClasses()) {
			debug("Renaming class " + cg.getClassName());
			JavaClass jc = renameJavaClass(cg.getJavaClass());
			finalClasses.add(jc);
		}
	}

	public void save() {
		Application finalApp = new Application();

		for (JavaClass jc : finalClasses)
			finalApp.setJavaClass(jc);

		finalApp.dumpJar(outFile);
	}

	public JavaClass renameJavaClass(JavaClass jc) {
		ConstantPool cp = jc.getConstantPool();
		Constant[] cons = cp.getConstantPool();

		for (int i = 0; i < cons.length; i++) {
			if (!(cons[i] instanceof ConstantUtf8))
				continue;

			ConstantUtf8 cutf = (ConstantUtf8)cons[i];
			for (Map.Entry<String, String> entry : classes.entrySet()) {
				String oldName = entry.getKey();
				String newName = entry.getValue();
				String val = cutf.getBytes();

				if (val.contains(oldName)) { 
					if (shouldAlter(oldName, val)) {
						debug("  - " + oldName + " -> " + newName + " in " + val);
						cutf.setBytes(val.replace(oldName, newName));
						debug("    - new value: " + cutf.getBytes());
						cons[i] = cutf;
					} else
						debug("  - shouldn't alter: " + val);
				}
			}
		}

		cp.setConstantPool(cons);
		jc.setConstantPool(cp);
		return jc;
	}

	public boolean shouldAlter(String needle, String haystack) {
		return Pattern.matches(".*L(?<!rs/)" + needle + ".*", haystack)
				|| haystack.equals(needle);
	}

	public static void debug(String s) {
		if (verbose)
			System.out.println(s);
	}

	public static void fatal(String err) {
		System.err.println(err);
		System.exit(1);
	}
}
