import java.io.File;
import java.util.TreeMap;
import java.util.Map;
import org.pd.parser.*;

public class Parser extends RuleReader {
	private TreeMap<String, String> classes;
	private TreeMap<String, String> constants;

	private static boolean verbose = false;

	public Parser(File defFile, File inDir) {
		super(defFile, inDir);

		classes = new TreeMap<String, String>();
		constants = new TreeMap<String, String>();
	}

	public static void main(String[] args) {
		Parser p;

		if (args.length < 2)
			fatal("usage: Parser deffile inputdir");

		verbose = System.getenv("VERBOSE") != null;
		p = new Parser(new File(args[0]), new File(args[1]));
		p.parse();
		p.dump();
	}

	public void handleExtraction(String type, String name, String matched) {
		if (verbose)
			System.err.println(name + "=" + matched);
		if (type.equalsIgnoreCase("class"))
			classes.put(name, matched);
		else if (type.equalsIgnoreCase("constant"))
			constants.put(name, matched);
		else
			fatal("Unsupported extraction type: " + type);
	}

	public void dump() {
		System.out.println("[classes]");
		for (Map.Entry<String, String> e : classes.entrySet())
			System.out.println(e.getKey() + "=" + e.getValue());

		System.out.println("");
		System.out.println("[constants]");
		for (Map.Entry<String, String> e : constants.entrySet())
			System.out.println(e.getKey() + "=" + e.getValue());
	}

	public String findVariable(String var) {
		if (classes.containsKey(var))
			return classes.get(var);
		else if (constants.containsKey(var))
			return constants.get(var);
		else
			return getVariables().get(var);
	}

	public boolean isClassRenamed(String name) {
		return classes.containsValue(name);
	}

	public static void fatal(String err) {
		System.err.println(err);
		System.exit(1);
	}
}
