#!/bin/sh

BASE=${BASE:=$PWD}
SAVEDIR=${SAVEDIR:="$BASE/final"}
JAD=${JAD:="jad"}

die() {
	echo $*
	exit 1
}

prepare() {
	[[ -r $SRCJAR ]] || die "invalid srcjar: $SRCJAR"
	[[ -r $BINJAR ]] || die "invalid binjar: $BINJAR"

	mkdir -p tmp/{src,bin,jad} || die "couldn't make tmp directories"
	mkdir -p $SAVEDIR/{renamed,aryan}
	mkdir -p $SAVEDIR/renamed/{src,bin}

	cp $BINJAR tmp/bin
	cp $SRCJAR tmp/src && (cd tmp/src;jar xf *.jar)
}

fixbc() {
	(rm -f tmp/jad/{INPUT,OUTPUT}/rs/ObjectManager.*;
	 mkdir -p tmp/bc;
	 cd tmp/bc;
	 cp ../packaged.jar ./runescape.jar;
	 jar xf runescape.jar;
	 cp rs/ObjectManager.class ../jad/OUTPUT/rs;
	 cp rs/ObjectManager.class ../jad/INPUT/rs;
	 cd ../..;
	 rm -rf tmp/bc)
}

mkjar() {
	(cd tmp/jad/INPUT;
	 jar cf rs${vers}.jar rs/*.class rs/sign/*.class;
	 chmod -x rs${vers}.jar;
	 cp rs${vers}.jar $SAVEDIR/renamed/bin)
	(cd tmp/jad/OUTPUT;
	 jar cf RuneScape${vers}.jar rs/*.java rs/*.class rs/sign/*.java;
	 chmod -x RuneScape${vers}.jar;
	 cp RuneScape${vers}.jar $SAVEDIR/renamed/src)
}

factor() {
	echo "Running factor suite"

	echo " -- Running parser"
	(cd parser;
	 sh run.sh rules/main ../tmp/src/>../renamer/conf)
	[[ $? -eq 0 ]] || die "Error running parser" 

	echo " -- Running renamer"
	(cd renamer;
	 sh run.sh ./conf ../tmp/bin/*.jar ../tmp/renamed.jar)

	echo " -- Packaging"
	(cd packager;
	 sh run.sh ../tmp/renamed.jar ../tmp/packaged.jar)

	echo " -- Decompiling"
	(cd tmp/jad;
	 rm -rf INPUT/* OUTPUT/*;
	 mkdir -p INPUT OUTPUT/sign;
	 cp ../packaged.jar ./INPUT;
	 cd INPUT;
	 jar xf packaged.jar;
	 ${JAD} -s java -d ../OUTPUT/rs rs/*.class >/dev/null 2>&1;
	 ${JAD} -s java -d ../OUTPUT/rs/sign rs/sign/*.class >/dev/null 2>&1)

	echo " -- Fixing bad class"
	fixbc
	dos2unix tmp/jad/OUTPUT/rs/*.java >/dev/null 2>&1

	echo " -- Making final jar files"
	mkjar
}

aryan() {
	echo "Preparing for use with Aryan"
	rm -rf tmp/aryan
	mkdir -p tmp/aryan/{INPUT,OUTPUT,FINAL}
	mkdir -p tmp/aryan/OUTPUT/rs

	f=$SAVEDIR/renamed/src/RuneScape${vers}.jar
	[[ -r $f ]] || die "Can't access renamed source jar: $f"

	cp $f tmp/aryan/INPUT
	(cd tmp/aryan/INPUT;
	 jar xf RuneScape${vers}.jar)

	(cd actions;
	 sh run.sh rules/actions ../tmp/aryan/INPUT > ../tmp/aryan/OUTPUT/rs/ActionConstants.java)
	[[ $? -eq 0 ]] || exit 1
	(cd massager;
	 sh run.sh rules/aryan ../tmp/aryan/INPUT ../tmp/aryan/OUTPUT/rs)
	[[ $? -eq 0 ]] || exit 1
	(cd tools;
	 perl middleman.pl ../tmp/aryan/OUTPUT/rs/client.java ../tmp/aryan/OUTPUT/rs)
	[[ $? -eq 0 ]] || exit 1

	cp -R tmp/aryan/INPUT/* tmp/aryan/FINAL
	cp -R tmp/aryan/OUTPUT/* tmp/aryan/FINAL

	(cd tmp/aryan/FINAL;
	 jar cf aryan${vers}.jar rs/*.java rs/*.class rs/sign/*.java;
	 cp aryan${vers}.jar $SAVEDIR/aryan)
}

usage() {
	echo "usage: $0 [factor|aryan|all] srcjar binjar"
	exit 1
}

[[ $# -eq 3 ]] || usage
ACTION=$1
SRCJAR=$2
BINJAR=$3

vers="_${VERSION}"
[[ "$vers" == "_" ]] && vers=""

prepare
case "$ACTION" in
	"factor") factor;;
	"aryan") aryan;;
	"all")
		factor;
		aryan;;
	*) usage;;
esac
